/* TestShl -	The program returns back errorlevels based upon if a Novell
	driver is loaded in memory.  The program can detect the
	following drivers and will return the appropriate errorlevel
	if they are not loaded:

	Driver			Errorlevel
	------			----------
	LSL			1 \__ All are board level drivers
	HLS MPD			1 /
	Clarkson PKT DRVR	1/
	ODI driver		2 * Not available yet
	IPX			3
	NETx			4
	TMBI/TMBI2		5

	To run, the following parameters are supported:
	L - LSL			I - IPX
	O - ODI			N - NETx
	C - Clarkson DRVR	T - TMBI/TMBI2
	M - HLS MPD		A - All drivers checked

	For the ALL drivers test, the program will return the
	errorlevel for the first driver it cannot detect.  i.e.
	if only LSL and ODI are loaded the errorlevel returned would
	be 3 (IPX not loaded).

Thanks:	The program is based on code from:
		PC INTERRUPTS  by R. Brown & J. Kyle
		NetWare Systems Calls
		PDS Bullets, volume 4, number 7, page 7
	Several people assisted with the development by pointing
	me in the right direction:
		Gecko (100023,342)
		Steve Meyer (76517,3117)

Legal:	This unmodified source file and its executable form may be
	used and redistributed freely.  The source may be modified, and
	the source or executable versions built from the modified source
	may be used and redistributed, provided that this notice and the
	copyright notice displayed by the executable remains intact, and
	provided that the executable displays an additional message
	indicating that it has been modified, and by whom.

	Ken McKinlay releases this software "as is", with no express or
	implied warranty, including, but not limited to, the implied
	warranties of merchantability and fitness for a particular
	program.  (Otherwise known as use at own risk.)

	Please send bug reports to Ken McKinlay at CompuServe 71531,3150.
*/

#define	DOS_INT		0x21	/* DOS function call interrupt */
#define	MULTIPLEX	0x2f	/* Multiplex interrupt */

#include	<dos.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

int	TestClarkson(void),
	TestIpx(void),
	TestLsl(void),
	TestMpd(void),
	TestNet(void),
	TestOdi(void),
	TestTmbi(void);
void	Help(void);

union	REGS	registers;
struct	SREGS	segments;

static	char	buffer[9];

void	main(int argc, char *argv[])
{
	int	stringPointer=0;
	int	errorLevel=0;

	/* Check if any arguments were specified at the command line */
	if (argc--)
	{
		/* If a slash or dash is the first character of the parameter
			then increment the pointer to look at the next character */
		if ((argv[1][1] == '-') || (argv[1][1] == '\\') || (argv[1][1] == '/'))
			stringPointer++;

		/* Check the parameters and process if it matches, otherwise
			display the help list */
		switch (argv[1][stringPointer])
		{
			case	'A':
			case	'a':	if ((errorLevel=TestLsl()) == 0)
					{
						if ((errorLevel=TestOdi()) != 0)
							break;
					}
					else
					{
						if ((errorLevel=TestMpd()) != 0)
							if ((errorLevel=TestClarkson()) != 0)
								break;
					}
					if ((errorLevel=TestIpx()) != 0)
						break;
					if ((errorLevel=TestNet()) != 0)
						break;
					if ((errorLevel=TestTmbi()) != 0)
						break;
			case	'C':
			case	'c':	errorLevel=TestClarkson();
					break;
			case	'I':
			case	'i':	errorLevel=TestIpx();
					break;
			case	'L':
			case	'l':	errorLevel=TestLsl();
					break;
			case	'M':
			case	'm':	errorLevel=TestMpd();
					break;
			case	'N':
			case	'n':	errorLevel=TestNet();
					break;
			case	'O':
			case	'o':	errorLevel=TestOdi();
					break;
			case	'T':
			case	't':	errorLevel=TestTmbi();
					break;
			default:	Help();
		}

		exit(errorLevel);
	}
	else
		Help();
}

/* Display the help list and exit with an E/L = 255 */
void	Help(void)
{
	printf("TestShl  v1.01 - Ken McKinlay (71531,3150)  (c) 1992\n");
	printf("         This program is released into the Public Domain.\n\n");
	printf("This program returns back errorlevels based on if a Novell\n");
	printf("driver is loaded in memory.  The program can detect the following\n");
	printf("drivers and will return the appropriate errorlevel if they are\n");
	printf("not loaded:\n\n");
	printf("Parameter  Driver      Errorlevel\n");
	printf("---------  ------      ----------\n");
	printf("    L      LSL         1 \\____ All are board level drivers.\n");
	printf("    M      HLS MPD     1 /\n");
	printf("    C      Clarkson    1/\n");
	printf("    O      ODI         2    ** not available yet\n");
	printf("    I      IPX         3\n");
	printf("    N      NETx        4\n");
	printf("    T      TMBI/TMBI2  5\n");
	printf("    A      All drivers\n\n");
	printf("For the ALL drivers test, the program will return the errorlevel for\n");
	printf("the first driver it cannot detect.  i.e. if only LSL and ODI are loaded\n");
	printf("the errorlevel returned would be 3 (IPX not loaded).\n");

	exit(255);
}

/********
 If the Clarkson packet driver is not loaded, return E/L = 1
 The Clarkson packet driver initializes itself and links to an interrupt
 specified by the user or by default to 0x60.  It places the signature
 'PKT DRVR' 3 bytes after the address pointed to by the vector.
********/

int	TestClarkson(void)
{
	char	far	*address;
	int	intNumber;
	int	returnValue=1;
	void	(interrupt far *intHandler)();

	/* Cycle through the possible intNumber values for the Clarkson driver */
	for (intNumber=0x60; intNumber < 0x80; intNumber++)
	{
		intHandler=_dos_getvect((unsigned)intNumber);
		address=(void far *)buffer;
		movedata(FP_SEG(intHandler),FP_OFF(intHandler)+3,FP_SEG(address),FP_OFF(address),8);
		buffer[9]='\0';
		if (strcmp(buffer,"PKT DRVR")==0)
		{
			returnValue=0;
			break;
		}
	}
	return(returnValue);
}

/********
 If the LSL driver is not loaded, return E/L = 1
 The LSL driver initializes itself and links to the Multiplex INT between
 0xc0 and 0xff.  It places a signature, "LINKSUP$"  that is pointed to
 by ES:SI.
********/

int	TestLsl(void)
{
	int	i;
	char	far	*address;
	int	returnValue=1;

	/* Cycle through the possible multiplex values for LSL */
	for (i=0xc0; i <= 0xff; i++)
	{
		registers.h.ah=(unsigned char)i;
		registers.h.al=0x00;
		int86x(MULTIPLEX,&registers,&registers,&segments);
		if (registers.h.al==0xff)
		{
			address=(void far *)buffer;
			movedata(segments.es,registers.x.si,FP_SEG(address),FP_OFF(address),8);
			buffer[9]='\0';
			if (strcmp(buffer,"LINKSUP$")==0)
			{
				returnValue=0;
				break;
			}
		}		
	}
	return(returnValue);
}

/********
 If the Hughes LAN Systems MPD driver is not loaded, return E/L = 1.
 The MLS MPD driver initializes itself and links to the Multiplex INT
 at 0x80.  It places a signature, "PKT DRVR"  that is pointed to by DS:SI.
********/

int	TestMpd(void)
{
	char	far	*address;
	int	returnValue=1;

	registers.x.ax=0x8000;
	int86x(MULTIPLEX,&registers,&registers,&segments);
	if (registers.h.al==0xff)
	{
		address=(void far *)buffer;
		movedata(segments.ds,registers.x.si,FP_SEG(address),FP_OFF(address),8);
		buffer[9]='\0';
		if (strcmp(buffer,"PKT DRVR")==0)
			returnValue=0;
	}
	return(returnValue);
}

/* If the ODI driver is not loaded, return E/L = 2.  At the present time,
   the code to detect whether the ODI driver is loaded is not available.
   It always returns 0. */
int	TestOdi(void)
{
	return(0);
}

/* If the IPX driver is not loaded, return E/L = 3 */
int	TestIpx(void)
{
	registers.x.ax=0x7a00;
	int86x(MULTIPLEX,&registers,&registers,&segments);
	return((registers.h.al==0xff) ? 0:3);
}

/* If the NETx driver is not loaded, return E/L = 4 */
int	TestNet(void)
{
	registers.x.ax=0xdc00;		/* Get connection number */
	int86x(DOS_INT,&registers,&registers,&segments);
	return((registers.h.al==0x00) ? 4:0);
}

/* If the TMBI/TMBI2 driver is not loaded, return E/L = 5 */
int	TestTmbi(void)
{
	registers.x.dx=0x00;
	registers.x.ax=0x7a10;
	int86x(MULTIPLEX,&registers,&registers,&segments);
	return((registers.x.dx==0x00) ? 5:0);
}
